from setuptools import setup

setup(
    name='kgreb',
    version='v0.0.1',
    packages=['kgreb', 'kgreb.vle', 'kgreb.util'],
    url='https://github.com/BigPotatoPizzaHey/kgreb/',
    license='',
    author='BigPotatoPizzaHey',
    author_email='',
    description=''
)
